=begin
#Merged swaggers

#All happy services merged together 

OpenAPI spec version: 2.7.0

Generated by: https://github.com/swagger-api/swagger-codegen.git

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

=end

require 'date'

module HappyCo
  # * Reports are the end artifact of one or sometimes two inspections. They are generated from inspection(s) at a point in time but do not update if the underlying inspection updates. They are self contained once generated. They are intended to be similar to a printed pieice of paper in that they can't be edited but they can be signed at any time.
  class V1Report
    # * ID of report.
    attr_accessor :id

    # * Public URL can be used to share the inspection. It contains a UUID to provide security.
    attr_accessor :public_url

    # * Name of the asset that was inspected. This is typically the address of the unit or house that was inspected.
    attr_accessor :name

    # * Title of the report. This can be overriden by the user in the report settings but typically comes from the template name used to perform the inspection.
    attr_accessor :title

    # * The revision of the report. This will increment when a change occurs. Reports are limited to changes being archive/unarchive operations or new signatures being added.
    attr_accessor :revision

    # * Account ID that the report belongs to. For custom integrations (i.e. not integration partners) it may be omitted.
    attr_accessor :account_id

    # * FolderID references the folder that contains the inspections used to generate the report.
    attr_accessor :folder_id

    # * Asset ID references the parent that inspection is attached to. This field must reference a non archived asset in the same folder as the template referenced below.
    attr_accessor :asset_id

    # * Inspection IDs references one or more inspection(s) used to generate the report.
    attr_accessor :inspection_ids

    # * Generated at is the time (as reported by the mobile device) that the report was generated at.
    attr_accessor :generated_at

    # * Finalized is the time (as reported by the mobile device) that the report was locked. Until the report is locked the display settings can be changed. Signing a report locks it immediately.
    attr_accessor :finalized_at

    # * Inspector ID references the user who completed the most recent inspection used to generate the report.
    attr_accessor :inspector_id

    # * Inspector name references the user who completed the most recent inspection used to generate the report.
    attr_accessor :inspector_name

    attr_accessor :data

    # * A map of signatories added to the report. Each key in the map only needs to be unique to this report.
    attr_accessor :signatories


    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        :'id' => :'id',
        :'public_url' => :'public_url',
        :'name' => :'name',
        :'title' => :'title',
        :'revision' => :'revision',
        :'account_id' => :'account_id',
        :'folder_id' => :'folder_id',
        :'asset_id' => :'asset_id',
        :'inspection_ids' => :'inspection_ids',
        :'generated_at' => :'generated_at',
        :'finalized_at' => :'finalized_at',
        :'inspector_id' => :'inspector_id',
        :'inspector_name' => :'inspector_name',
        :'data' => :'data',
        :'signatories' => :'signatories'
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        :'id' => :'V1IntegrationID',
        :'public_url' => :'String',
        :'name' => :'String',
        :'title' => :'String',
        :'revision' => :'Integer',
        :'account_id' => :'V1IntegrationID',
        :'folder_id' => :'V1IntegrationID',
        :'asset_id' => :'V1IntegrationID',
        :'inspection_ids' => :'Array<V1IntegrationID>',
        :'generated_at' => :'V1DateTime',
        :'finalized_at' => :'V1DateTime',
        :'inspector_id' => :'V1IntegrationID',
        :'inspector_name' => :'String',
        :'data' => :'String',
        :'signatories' => :'Hash<String, V1Signatory>'
      }
    end

    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}){|(k,v), h| h[k.to_sym] = v}

      if attributes.has_key?(:'id')
        self.id = attributes[:'id']
      end

      if attributes.has_key?(:'public_url')
        self.public_url = attributes[:'public_url']
      end

      if attributes.has_key?(:'name')
        self.name = attributes[:'name']
      end

      if attributes.has_key?(:'title')
        self.title = attributes[:'title']
      end

      if attributes.has_key?(:'revision')
        self.revision = attributes[:'revision']
      end

      if attributes.has_key?(:'account_id')
        self.account_id = attributes[:'account_id']
      end

      if attributes.has_key?(:'folder_id')
        self.folder_id = attributes[:'folder_id']
      end

      if attributes.has_key?(:'asset_id')
        self.asset_id = attributes[:'asset_id']
      end

      if attributes.has_key?(:'inspection_ids')
        if (value = attributes[:'inspection_ids']).is_a?(Array)
          self.inspection_ids = value
        end
      end

      if attributes.has_key?(:'generated_at')
        self.generated_at = attributes[:'generated_at']
      end

      if attributes.has_key?(:'finalized_at')
        self.finalized_at = attributes[:'finalized_at']
      end

      if attributes.has_key?(:'inspector_id')
        self.inspector_id = attributes[:'inspector_id']
      end

      if attributes.has_key?(:'inspector_name')
        self.inspector_name = attributes[:'inspector_name']
      end

      if attributes.has_key?(:'data')
        self.data = attributes[:'data']
      end

      if attributes.has_key?(:'signatories')
        if (value = attributes[:'signatories']).is_a?(Array)
          self.signatories = value
        end
      end

    end

    # Show invalid properties with the reasons. Usually used together with valid?
    # @return Array for valid properies with the reasons
    def list_invalid_properties
      invalid_properties = Array.new
      return invalid_properties
    end

    # Check to see if the all the properties in the model are valid
    # @return true if the model is valid
    def valid?
      return true
    end

    # Checks equality by comparing each attribute.
    # @param [Object] Object to be compared
    def ==(o)
      return true if self.equal?(o)
      self.class == o.class &&
          id == o.id &&
          public_url == o.public_url &&
          name == o.name &&
          title == o.title &&
          revision == o.revision &&
          account_id == o.account_id &&
          folder_id == o.folder_id &&
          asset_id == o.asset_id &&
          inspection_ids == o.inspection_ids &&
          generated_at == o.generated_at &&
          finalized_at == o.finalized_at &&
          inspector_id == o.inspector_id &&
          inspector_name == o.inspector_name &&
          data == o.data &&
          signatories == o.signatories
    end

    # @see the `==` method
    # @param [Object] Object to be compared
    def eql?(o)
      self == o
    end

    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [id, public_url, name, title, revision, account_id, folder_id, asset_id, inspection_ids, generated_at, finalized_at, inspector_id, inspector_name, data, signatories].hash
    end

    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)
      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            self.send("#{key}=", attributes[self.class.attribute_map[key]].map{ |v| _deserialize($1, v) } )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          self.send("#{key}=", _deserialize(type, attributes[self.class.attribute_map[key]]))
        end # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end

    # Deserializes the data based on type
    # @param string type Data type
    # @param string value Value to be deserialized
    # @return [Object] Deserialized data
    def _deserialize(type, value)
      case type.to_sym
      when :DateTime
        DateTime.parse(value)
      when :Date
        Date.parse(value)
      when :String
        value.to_s
      when :Integer
        value.to_i
      when :Float
        value.to_f
      when :BOOLEAN
        if value.to_s =~ /^(true|t|yes|y|1)$/i
          true
        else
          false
        end
      when :Object
        # generic object (usually a Hash), return directly
        value
      when /\AArray<(?<inner_type>.+)>\z/
        inner_type = Regexp.last_match[:inner_type]
        value.map { |v| _deserialize(inner_type, v) }
      when /\AHash<(?<k_type>.+), (?<v_type>.+)>\z/
        k_type = Regexp.last_match[:k_type]
        v_type = Regexp.last_match[:v_type]
        {}.tap do |hash|
          value.each do |k, v|
            hash[_deserialize(k_type, k)] = _deserialize(v_type, v)
          end
        end
      else # model
        temp_model = HappyCo.const_get(type).new
        temp_model.build_from_hash(value)
      end
    end

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # to_body is an alias to to_hash (backward compatibility)
    # @return [Hash] Returns the object in the form of hash
    def to_body
      to_hash
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = self.send(attr)
        next if value.nil?
        hash[param] = _to_hash(value)
      end
      hash
    end

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map{ |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end

  end

end
